//Комментирование Cinema от 09/11/2020.
/*
  В библиотек AD770X, в файле AD770x.cpp есть строка инициализации в самом конце.
  Её, возможно, придётся переписать так:
  init(channel, CLK_DIV_1, BIPOLAR, GAIN_4, UPDATE_RATE_200);
*/


//подключенные библиотеки
#include <Wire.h>
#include <LiquidCrystal_I2C.h>
LiquidCrystal_I2C lcd(0x27, 16, 2);
#include <string.h>
#include <AD770X.h>
AD770X ad7706(2.5);

//пины подключения
byte buzzerPin = 4;                   //пин пищалки
#define RelayPin1 6                   //пин управления ВИ
#define RelayPin2 5                   //пин управления НИ
#define EnableTop 2                   //LED состояния ВИ (ON или OFF) 
#define EnableBottom 3                //LED состояния НИ (ON или OFF)
#define DRDY 9                        //пин подключения вывода 12(DATA READY)АЦП AD7705

//переменные
double R1, R2;                        //переменные для хранения t°С НИ(R1) и ВИ(R2), считанных AD7705
byte LevelTop = 0x00;                 //мощность ВИ начальная
byte LevelBottom = 0x00;              //мощность НИ начальная
double Temp_Top = 0;                  //t°C ВИ начальная
double Temp_Bottom = 0;               //t°C НИ начальная
int Temp1, Temp2, Temp11, Temp12;     //переменные для
byte EnableLed = 0xFF;                //подсветка дисплея "ON"
const byte DESIRED_WIDTH = 3;         //длина блока данных для LCD
char command_in[6];

void setup() {
  Serial.begin(9600);                 //активируем функцию Serial

  //приветствие на LCD
  lcd.begin();                        //активируем LCD
  lcd.clear();                        //очищаем дисплей
  lcd.backlight();                    //устанавливаем состояние подсветки
  lcd.setCursor(1, 0);
  lcd.print("SOLDER STATION");
  lcd.setCursor(5, 1);
  lcd.print("v2.1.2");

  //мелодия приветствия
  SongHello ();

  delay(3000);                        //показываем приветствие 3 секунды
  lcd.clear();                        //очищаем дисплей

  //задаём состояние пинам подключения
  pinMode(RelayPin1, OUTPUT);
  pinMode(RelayPin2, OUTPUT);
  pinMode(EnableTop, OUTPUT);
  pinMode(EnableBottom, OUTPUT);
  digitalWrite(EnableTop, LOW);
  digitalWrite(EnableBottom, LOW);
  analogWrite(RelayPin1, LevelTop);
  analogWrite(RelayPin2, LevelBottom);
  ad7706.reset();
  ad7706.init(AD770X::CHN_AIN1);
  ad7706.init(AD770X::CHN_AIN2);
}

//параметры строки для вывода данных
String addZeroes(const String & src, byte pad)
{
  if (src.length() >= pad)
    return src;
  String result;
  byte add = pad - src.length();
  while (add--)
    result += "0";
  result += src;
  return result;
}

//основной цикл програмы
void loop() {
  //читаем AD7705
  while ( !digitalRead(DRDY) );
  R1 = ad7706.readADResult(AD770X::CHN_AIN1);
  double Temp_Bottom = (R1 - 1) / 0.00392;
  //delay(1);  // раскоментировать, если читает температуру некорректно
  R2 = ad7706.readADResult(AD770X::CHN_AIN2);
  double Temp_Top = (R2 - 1) / 0.00392 - 10;

  //работа с буфером обмена (общение с софтом для ПК)
  int i = 0;
  int buffer[6];
  if (Serial.available()) {
    delay(100);
    while ( Serial.available() && i < 7) {
      buffer[i++] = Serial.read();
    }
    buffer[i++] = '\0';
  }
  if (i > 0) {
    if (buffer[0] == 0xAA) {
      LevelTop = buffer[1];
      LevelBottom = buffer[2];
      EnableLed = buffer[3];
    }
    Temp1 = int(Temp_Top);
    Temp11 = 0;
    if (Temp1 > 255) {
      Temp11 = Temp1 - 255;
      Temp1 = 255;
    }
    Temp2 = int(Temp_Bottom);
    Temp12 = 0;
    if (Temp2 > 255) {
      Temp12 = Temp2 - 255;
      Temp2 = 255;
    }
    Serial.write(170);
    Serial.write(Temp1);
    Serial.write(Temp11);
    Serial.write(Temp2);
    Serial.write(Temp12);
    Serial.write(255);
  }
  if (LevelTop == 0x00) {
    digitalWrite(EnableTop, LOW);
  }
  if (LevelTop != 0x00) {
    digitalWrite(EnableTop, HIGH);
  }
  if (LevelBottom == 0x00) {
    digitalWrite(EnableBottom, LOW);
  }
  if (LevelBottom != 0x00) {
    digitalWrite(EnableBottom, HIGH);
  }
  analogWrite(RelayPin1, LevelTop);
  analogWrite(RelayPin2, LevelBottom);

  //вывод на LCD данных о t°C и о мощности ВИ и НИ
  lcd.setCursor(1, 0);
  lcd.print("TH:");
  lcd.setCursor(1, 1);
  lcd.print("BH:");
  lcd.setCursor(8, 0);
  lcd.print("PWR:");
  lcd.setCursor(8, 1);
  lcd.print("PWR:");
  lcd.setCursor(4, 0);
  lcd.print(addZeroes(String(int(Temp_Top)), DESIRED_WIDTH));
  lcd.setCursor(4, 1);
  lcd.print(addZeroes(String(int(Temp_Bottom)), DESIRED_WIDTH));
  lcd.setCursor(12, 0);
  lcd.print(addZeroes(String(int(LevelTop)), DESIRED_WIDTH));
  lcd.setCursor(12, 1);
  lcd.print(addZeroes(String(int(LevelBottom)), DESIRED_WIDTH));

  //управление  ON/OFF подсветкой LCD
  if (EnableLed == 0x00) {
    lcd.noBacklight();
  }
  if (EnableLed == 0xFF) {
    lcd.backlight();
  }
}

//---Мелодия приветствия---//
void SongHello () {
  tone(buzzerPin, 523);
  delay(200);
  tone(buzzerPin, 659);
  delay(200);
  tone(buzzerPin, 784);
  delay(200);
  tone(buzzerPin, 1046);
  delay(200);
  noTone(buzzerPin);
}
